/*
 *	analy.cc
 */

/*	Copyright (C) 1999, 2000, 2001 Sebastian Biallas (sb@web-productions.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "analy.h"
#include "analy_names.h"
#include "codeanaly.h"
#include "dataanaly.h"
#include "fs.h"
#include "global.h"
#include "htctrl.h"
#include "htdebug.h"
#include "httag.h"
#include "store.h"
#include "tools.h"
#include "x86dis.h"

analyser *theanaly=NULL;
char* (*analy_addr_sym_val_func)(CPU_ADDR addr)=NULL;
analyser *testanaly=NULL;

//#undef dprintf
//#define dprintf(msg...) printf(##msg)


/*
 *
 */
void	analyser::init()
{
	testanaly = this;

	active = false;
	addr_queue = new tqueue();
	addr_queue->init();
	addrs = NULL;
	labels = NULL;
	set_addr_tree_optimize_threshold(1000);
	set_label_tree_optimize_threshold(1000);
	cur_addr_ops = cur_label_ops = 1;
	cur_func = NULL;
	ops_parsed = 0;
	next_explored = first_explored = last_explored = INVALID_ADDR;
	explored = new	area();
	explored->init();
	init_codeanalyser();
	init_dataanalyser();
	disasm = NULL;
	analy_disasm = NULL;
	init_unasm();
	mode = ANALY_TRANSLATE_SYMBOLS;
#ifdef ANALY_TIMINGS
	timer = new_timer();
#endif
}


void loadaddrs(ht_stream_filter *f, taddr **addr, int level, int *left)
{
/*	if ((*left)<=0) {
		*addr = NULL;
		return;
	}

	*addr = (taddr *) calloc(sizeof(taddr), 1);


	if ((level > 1) && (*left > 1)) loadaddrs(f, &(*addr)->left, level / 2, left);

	f->read(&(*addr)->addr, sizeof(ADDR));
//+     fstrget(f, &(*addr)->label);
	// xrefs
	(*addr)->xreflist = NULL;
	txref	**xref = &(*addr)->xreflist;
	int		count = 0;
	f->read(&count, sizeof(int));
	while (count--) {
		f->read(&(*xref)->addr, sizeof(ADDR));
		f->read(&(*xref)->type, sizeof(txreftype));
		xref = &(*xref)->next;
	}
	xref = NULL;
	// comments
	(*addr)->comments = NULL;
	tcomment	**com = &(*addr)->comments;
	count = 0;
	f->read(&count, sizeof(int));
	while (count--) {
		fstrget(f, &(*com)->text);
		com = &(*com)->next;
	}
	com = NULL;

//	f->read(&(*addr)->type, sizeof(taddrtype));

	(*left)--;

	if ((level > 1) && (*left > 1)) loadaddrs(f, &(*addr)->right, level / 2, left);
	*/
}
/*
 *
 */
int analyser::load(ht_stream_filter *f)
{
	return 0;
/*	addr_queue->load(f);
	f->read(&ops_parsed, sizeof(int));
	f->read(&active, sizeof(bool));
	f->read(&next_explored, sizeof(ADDR));
	f->read(&first_explored, sizeof(ADDR));
	f->read(&last_explored, sizeof(ADDR));
	f->read(mode
	explored->load(f);
	int count;
	f->read(&count, sizeof(int));
	loadaddrs(f, &addrs, count, &count);
	code->load(f);
	data->load(f);
//	unasm->load(f);
	return 0;*/
}

/*
 *
 */
void	analyser::done()
{
	setactive(false);
/*	int testtimer = new_timer();
	printf("start %d %d \n", getaddrcount(), getlabelcount());
	start_timer(testtimer);
	freeaddrs(addrs);
	freelabels(labels);
	stop_timer(testtimer);
	printf("stop \n");
	int msec = get_timer_msec(testtimer);
	delete_timer(testtimer);
	printf("--%d-- \n", msec);*/
//     char buf[100];
//     scanf("[a-zA-Z]", buf);
	if (addr_queue) {
		addr_queue->done();
		delete addr_queue;
	}
	if (explored) {
		explored->done();
		delete explored;
	}
	if (code) {
		code->done();
		delete code;
	}
	if (data) {
		data->done();
		delete data;
	}
	if (disasm) {
		disasm->done();
		delete disasm;
	}
#ifdef ANALY_TIMINGS
	delete_timer(timer);
#endif
}

/*
 *	addaddrlabel will never overwrite an existing label (like addlabel)
 */
bool analyser::addaddrlabel(ADDR Addr, char *Prefix, labeltype type, taddr *infunc=NULL)
{
	if (!validaddr(Addr, scvalid)) return false;


	char *prefix = label_prefix(Prefix);

	char	*label = (char	*)malloc(9+strlen(prefix));
	char	num[9];

	sprintf(num, HEX8FORMAT, Addr);
	strcpy(label, prefix);
	strcat(label, num);

	if (addlabel(Addr, label, type, infunc)) {
		return true;
	} else {
		free(label);
		return false;
	}
}

/*
 *
 */
void	analyser::addcomment(ADDR Addr, int line, char *c)
{
	// line 0 meens append (at the moment assume append every time ;-))

//	if (!validaddr(Addr, scvalid)) return;

	taddr *a = newaddr(Addr);

	tcomment *com = a->comments;

	if (com) {
		while (com->next) com = com->next;
		com->next	= (tcomment *)	malloc(sizeof(tcomment));
		com = com->next;
	} else {
		com = (tcomment *) malloc(sizeof(tcomment));
		a->comments = com;
	}

	com->next	= NULL;
	com->text	= c;

	dprintf("#(%08lx) comment `%s'\n", Addr, c);
}

/*
 * addlabel: create label if there isnt one
 *           fail if label exist on another address
 *
 */
bool analyser::addlabel(ADDR Addr, char *label, labeltype type, taddr *infunc=NULL)
{
	if (!validaddr(Addr, scvalid)) return false;

	taddr *a = newaddr(Addr);

	if (!a->label) {

		tlabel *l = newlabel(label, a, type, infunc);

		if (l->addr->addr != Addr) {
			// this label already exists at a different address
			return false;
		}

		a->label = l;

		switch (type) {
			case label_unknown:
				break;
			case label_func:
				data->setcodeaddrtype(a, dstfunction);
				break;
			case label_loc:
				data->setcodeaddrtype(a, dstlocation);
				break;
			case label_data:
				data->setaddrtype(a, dtunknowndata, 0, 0);
				break;
		}

		return true;

	} else {

		// adress already has a label
		return false;
	}
}

/*
 *
 */
void	analyser::addxref(ADDR from, ADDR to, txreftype action)
{
	if ((!validaddr(from, scvalid)) ||	(!validaddr(to, scvalid))) return;

	taddr	*a =	newaddr(from);
	txref	*x = a->xreflist;

	if (x) {
		while (x->next) x =	x->next;
		x->next =	(txref *)	malloc(sizeof(txref));
		x = x->next;
	} else {
		x = (txref *) malloc(sizeof(txref));
		a->xreflist = x;
	}

	x->next =	NULL;
	x->addr =	to;
	x->type = action;

	dprintf("xref %08lx->%08lx\n", from, to);
}

/*
 *
 */
void	analyser::assigncomment(ADDR Addr, int line, char *c)
{
	/* not really implemented */
	addcomment(Addr, line, c);
}

/*
 *
 */
bool analyser::assignlabel(ADDR Addr, char *label, labeltype type, taddr *infunc=NULL)
{
	if (!validaddr(Addr, scvalid)) return false;

	taddr *a = newaddr(Addr);

	tlabel *l = newlabel(label, a, type, infunc);
	if (l->addr->addr != Addr) {
		// label already exists at a different address
		return false;
	}

	switch (type) {
		case label_unknown:
			break;
		case label_func:
			data->setcodeaddrtype(a, dstfunction);
			break;
		case label_loc:
			data->setcodeaddrtype(a, dstlocation);
			break;
		case label_data:
			data->setaddrtype(a, dtunknowndata, 0, 0);
			break;
	}

	if (a->label) {
		// overwrite
		if (a->label != l) {
			// label has to be renamed
			disablelabel(a->label);
			a->label = l;
		}
	} else {
		a->label = l;
	}
	return true;
}

/*
 *
 */
void	analyser::beginanalysis(ADDR Addr)
{
	validaddr(0x12345678, scvalid);
	if (query_config(Q_DO_ANALYSIS)) {
		dprintf("################################\nAnalysis started at %08lx:\n", Addr);
		if (analy_disasm && disasm) {
			ops_parsed = 0;
			first_explored	= last_explored = Addr;
			cur_func = newaddr(Addr);
			data->setcodeaddrtype(cur_func, dstfunction);
			if (gotoaddr(Addr, Addr)) setactive(true);
		} else {
			dprintf("Analysis can't be started. No disassembler available.");
		}
	}
}

/*
 *
 */
void	*analyser::bufptr(ADDR Addr)
{
	// abstract
	return 0;
}

/*
 *
 */
bool	analyser::continueanalysis()
{
	void			*buf;
	OPCODE		*instr;
	int			len;
	tbranchtype	branch;

	if (!active) return	false;
#ifdef ANALY_TIMINGS
	start_timer(timer);
#endif
	do {
		if (!savetychangeaddr(addr)) {
#ifdef ANALY_TIMINGS
			stop_timer(timer);
#endif
			finish();
			return false;
		}

		buf = bufptr(addr);

		instr = disasm->decode((byte *)buf, maxopcodelength, mapaddr(addr));
		dprintf("opcode @%08lx [func: %08lx]: %s\n", addr, (cur_func) ? cur_func->addr : INVALID_ADDR, disasm->str(instr, 0));

		ops_parsed++;
		num_ops_parsed++;

		len = disasm->getsize(instr);
		last_explored += len;

		if (disasm->valid_insn(instr)) {
			branch = analy_disasm->isbranch(instr);
			if (branch != brnobranch) dobranch(branch, instr,	len); else {
				examineopcode(instr);
				addr	+= len;
			}
		} else {
			dprintf("invalid opcode @%08lx\n", addr);
//             log("invalid opcode at address %08lx\n", addr);
			newaddr(addr+len)->flags |= AF_FUNCTION_END;
			gotoaddr(INVALID_ADDR, INVALID_ADDR);
		}
	} while ((ops_parsed % MAX_OPS_PER_CONTINUE) !=0);
#ifdef ANALY_TIMINGS
	stop_timer(timer);
#endif
	notifyprogress(addr);
	return true;
}

/*
 *
 */
void analyser::continueanalysisat(ADDR Addr)
{
	if (!validcodeaddr(Addr)) return;
	if (query_config(Q_DO_ANALYSIS)) {
		dprintf("continueing analysis at %08lx\n", Addr);
		if (active) {
			pushaddr(Addr, Addr);
		} else {
			if (disasm) {
				beginanalysis(Addr);
			} else {
				dprintf("couldn't start analysis: no disasm available\n");
			}
		}
	}
}

/*
 *
 */
void	analyser::dataaccess(ADDR Addr, taccess	access)
{

	if (!validaddr(Addr, scvalid)) {
		char	msg[100];
		sprintf(msg, "access of invalid addr %08x at addr %08x", Addr, addr);
		log(msg);
		return;
	}

	dprintf("dataaccess of %08x\n", Addr);

	if (validcodeaddr(Addr)) {
		if (addaddrlabel(Addr, LPRFX_OFS, label_func)) {
			addcomment(Addr, 0, " ");
		}
		taddr *a = findaddr(Addr);
		assert(a);
		// test if Addr points to code
		if ((a->type.type == dtunknown) || (a->type.type == dtcode)) {
			// test if Addr points to valid code (test not yet complete)
			OPCODE *instr = disasm->decode((byte *)bufptr(Addr), maxopcodelength, mapaddr(Addr));
			if (disasm->valid_insn(instr)) {
				data->setcodeaddrtype(Addr, dstcunknown);
				pushaddr(Addr, Addr);
			}
		}
	} else {
		if (validaddr(Addr, scinitialized)) {
//               data->setaddrtype(Addr, dtunknowndata, 0, 0);
			addaddrlabel(Addr, LPRFX_DTA, label_data);
		} else {
//               data->setaddrtype(Addr, dtunknowndata, 0, 0);
			addaddrlabel(Addr, LPRFX_DTU, label_data);
		}
	}
}

/*
 *	disables label of an address and unassigns address' label
 */
void analyser::deletelabel(ADDR Addr)
{
	taddr *a = findaddr(Addr);
	if (a) {
		disablelabel(a->label);
		a->label = NULL;
	}
}

/*
 *	an disabled label will be overwritten as soon as possible,
 *   and never be returned nor saved.
 *	performed this way to preserve the labeltrees structure
 *
 */
void	analyser::disablelabel(tlabel *label)
{
	if (label) {
		label->addr = NULL;
	}
}

/*
 *
 */
void	analyser::dobranch(tbranchtype branch, OPCODE *opcode, int len)
{

	ADDR	branch_addr = analy_disasm->branchaddr(opcode);

	if (branch != brreturn) {
		if (!validcodeaddr(branch_addr)) {
			char	msg[100];
			sprintf(msg, "branch to invalid addr %08x from addr %08x", branch_addr, addr);
			log(msg);
			// a little hack
			examineopcode(opcode);
		}
	}
	switch (branch) {
		case	brjump:
			newaddr(addr+len)->flags |= AF_FUNCTION_END;

			addxref(branch_addr, addr, xrefjump);
			if (addaddrlabel(branch_addr, LPRFX_LOC, label_loc, cur_func)) {
				addcomment(branch_addr, 0, " ");
			}
			gotoaddr(branch_addr, INVALID_ADDR);
			break;
		case	brreturn:
			newaddr(addr+len)->flags |= AF_FUNCTION_END;

			gotoaddr(INVALID_ADDR, INVALID_ADDR);
			break;
		case	brcall:
			addxref(branch_addr, addr, xrefcall);
			if (addaddrlabel(branch_addr, LPRFX_SUB, label_func)) {
				addcomment(branch_addr, 0, " ");
				addcomment(branch_addr, 0, ";-----------------------");
				addcomment(branch_addr, 0, ";  S U B R O U T I N E");
				addcomment(branch_addr, 0, ";-----------------------");
				if (branch_addr==cur_func->addr) {
					addcomment(branch_addr, 0, ";  (recursive)");
				}
			}
			pushaddr(addr+len, cur_func->addr);
			gotoaddr(branch_addr, branch_addr);
			break;
		case	brjXX:
			addxref(branch_addr, addr, xrefjump);
			if (addaddrlabel(branch_addr, LPRFX_LOC, label_loc, cur_func)) {
				addcomment(branch_addr, 0, " ");
			}
			pushaddr(addr+len, cur_func->addr);
			gotoaddr(branch_addr, INVALID_ADDR);
			break;
		default:{}
			// stupid but neccessary
	}
}

/*
 *
 */
void	analyser::engage_codeanalyser()
{
	if (query_config(Q_ENGAGE_CODE_ANALYSER)) {
		dprintf("starting code analyser.\n");
	}
}

void analyserenum_addrs(taddr *addrs, ADDR at, taddr **addr)
{
	if ((at < addrs->addr) || (at == INVALID_ADDR)) {
		*addr = addrs;
		if (addrs->left) analyserenum_addrs(addrs->left, at, addr);
	} else /*if (at >= addrs->addr)*/ {
		if (addrs->right) analyserenum_addrs(addrs->right, at, addr);
	}
}

/*
 *
 */
taddr *analyser::enum_addrs(ADDR Addr)
{
	taddr *result = NULL;
	if (addrs) analyserenum_addrs(addrs, Addr, &result);
	return result;
}

void analyserenum_addrs_back(taddr *addrs, ADDR at, taddr **addr)
{
	if (at <= addrs->addr) {
		if (addrs->left) analyserenum_addrs_back(addrs->left, at, addr);
	} else /*if (at > addrs->addr)*/ {
		*addr = addrs;
		if (addrs->right) analyserenum_addrs_back(addrs->right, at, addr);
	}
}

/*
 *
 */
taddr *analyser::enum_addrs_back(ADDR Addr)
{
	taddr *result = NULL;
	if (addrs) analyserenum_addrs_back(addrs, Addr, &result);
	return result;
}

void analyserenum_labels(tlabel *labels, char *at, tlabel **label)
{
	int i = strcmp(at, labels->name);
	if (i < 0) {
		*label = labels;
		if (labels->left) analyserenum_labels(labels->left, at, label);
	} else /*if (i >= 0)*/ {
		if (labels->right) analyserenum_labels(labels->right, at, label);
	}
}

/*
 *   returns the (alphanumerically) next label to "at"
 *	or the first if "at" is NULL
 *	returns NULL if there is no preceding label
 *
 */
/* e.g.:
		tlabel *label = enum_labels(NULL);
		while (label) {
			printf("%d %08x %s\n", label->type, label->addr->addr, label->name);
			label = enum_labels(label->name);
		}
 */
tlabel *analyser::enum_labels(char *at)
{
	tlabel *result = NULL;
	if (!at) at="";
	if (labels) analyserenum_labels(labels, at, &result);

	// label with !addr mustnt be returned
	while ((result) && (!result->addr)) {
		analyserenum_labels(labels, result->name, &result);
	}

	return result;
}

void analyserenum_labels_back(tlabel *labels, char *at, tlabel **label)
{
	int i = strcmp(at, labels->name);
	if (i <= 0) {
		if (labels->left) analyserenum_labels_back(labels->left, at, label);
	} else /*if (i >= 0)*/ {
		*label = labels;
		if (labels->right) analyserenum_labels_back(labels->right, at, label);
	}
}

/*
 *
 */
tlabel *analyser::enum_labels_back(char *at)
{
	tlabel *result = NULL;
	// FIXME:
	if (!at) at="zzz";
	if (labels) analyserenum_labels_back(labels, at, &result);

	// labels with !addr mustnt be returned
	while ((result) && (!result->addr)) {
		analyserenum_labels_back(labels, result->name, &result);
	}

	return result;
}

/*
 *
 */
taddrtypetype analyser::examinedata(ADDR Addr)
{
	if ((validreadaddr(Addr)) && (validaddr(Addr, scinitialized))) {
		dprintf("examinating data @%08lx:\n", Addr);

	} else return dtunknown;
	return dtunknown;
}

/*
 *
 */
void	analyser::examineopcode(OPCODE *opcode)
{
	/*
	 *	very abstract
	 *
	 *		this function is called for each opcode parsed and has to
	 *		call dataaccess for EVERY addr opcode contains.
	 *
	 */
}

/*
 *	returns the fileofs of address Addr
 */
FILEADDR analyser::fileaddr(ADDR Addr)
{
	return INVALID_FILE_OFS;
}

/*
 *
 */
taddr *analyserfindaddr(taddr *addrs, ADDR Addr)
{
	if (addrs) {
		if (Addr < addrs->addr) return analyserfindaddr(addrs->left, Addr);
		if (Addr > addrs->addr) return analyserfindaddr(addrs->right, Addr);
		return addrs;
	}
	return NULL;
}

/*
 *
 */
taddr *analyser::findaddr(ADDR Addr)
{
	return analyserfindaddr(addrs, Addr);
}

/*
 *	findaddrfunc finds the function the Addr belongs to (if possible/applicable).
 */
taddr *analyser::findaddrfunc(ADDR Addr)
{
	taddr *a = findaddr(Addr);
	if (!a) a = enum_addrs_back(Addr);
	while ((a) && (!(a->flags & AF_FUNCTION_SET))) {
		if (a->flags & AF_FUNCTION_END) return NULL;
		a = enum_addrs_back(a->addr);
	}
	return (a) ? a->thisfunc : NULL;
}

/*
 *	findaddrlabel searches back to the last label
 */
taddr *analyser::findaddrlabel(ADDR Addr)
{
	taddr *a = findaddr(Addr);
	if (!a) a = enum_addrs_back(Addr);
	while ((a) && (!a->label)) {
		if (a->flags & AF_FUNCTION_END) return NULL;
		a = enum_addrs_back(a->addr);
	}
	return (a) ? a : NULL;
}

/*
 *
 */
tlabel *analyserfindlabel(tlabel *labels, char *label)
{
	if (labels) {
		int i=strcmp(label, labels->name);
		if (i < 0) return analyserfindlabel(labels->left, label);
		if (i > 0) return analyserfindlabel(labels->right, label);
		return labels->addr ? labels : NULL;
	}
	return NULL;
}

/*
 *
 */
tlabel *analyser::findlabel(char *label)
{
	return analyserfindlabel(labels, label);
}

/*
 *	called once every time the analyser has nothing more to do.
 */
void	analyser::finish()
{
	dprintf("the analyser finished (for now).\n");
	cur_func = NULL;
	addr = INVALID_ADDR;
	setactive(false);
}

/*
 *
 */
void analyser::freeaddr(taddr *Addr)
{
	if (Addr) {
		free(Addr);
		//...
	}
}

/*
 *
 */
void	analyser::freeaddrs(taddr *addrs)
{
	if (addrs) {
		freeaddrs(addrs->left);
		freeaddrs(addrs->right);
		freeaddr(addrs);
	}
}

/*
 *
 */
void analyser::freelabel(tlabel *label)
{
	if (label) {
//		if (label->name) free(label->name);
		free(label);
	}
}


/*
 *
 */
void analyser::freelabels(tlabel *labels)
{
	if (labels) {
		freelabels(labels->left);
		freelabels(labels->right);
		freelabel(labels);
	}
}

void analysergetaddrcount(taddr *addr, int *c)
{
	if (addr) {
		analysergetaddrcount(addr->right, c);
		(*c)++;
		analysergetaddrcount(addr->left, c);
	}
}

/*
 *
 */
int	analyser::getaddrcount()
{
	int c=0;
	analysergetaddrcount(addrs, &c);
	return c;
}


void analysergetlabelcount(tlabel *l, int *c)
{
	if (l) {
		analysergetlabelcount(l->left, c);
		if (l->addr) (*c)++;
		analysergetlabelcount(l->right, c);
	}
}

/*
 *
 */
int	analyser::getlabelcount()
{
	int c=0;
	analysergetlabelcount(labels, &c);
	return c;
}
/*
 *
 */
tlabel *analyser::getaddrlabel(ADDR Addr)
{
	taddr	*a = findaddr(Addr);

	return (a) ? a->label : NULL;
}

char *analyser::getaddrsectionname(ADDR Addr)
{
	return NULL;
}

/*
 *
 */
bool	analyser::gotoaddr(ADDR Addr, ADDR func)
{
	if (first_explored < last_explored) {
		dprintf("explored->add(%08lx - %08lx)\n", first_explored, last_explored);
		explored->add(first_explored, last_explored);
	}

	addr	= Addr;

	if ((explored->contains(addr)) || (!validcodeaddr(addr))) {
		dprintf("Address: %08lx Valid: %d Explored: %d\n", addr, validcodeaddr(addr), explored->contains(addr));
		do {
			if (!popaddr(&addr, &func)) {
				first_explored	= last_explored = INVALID_ADDR;
				addr	= INVALID_ADDR;
				return false;
			}

			dprintf("pop %08lx   (Valid: %d  Explored: %d)\n", addr, validcodeaddr(addr), explored->contains(addr));
		} while ((explored->contains(addr)) || (!validcodeaddr(addr)));
	}

	if (func!=INVALID_ADDR) {
		cur_func = newaddr(func);
		setaddrfunc(cur_func, cur_func);
	}

	next_explored = explored->findnext(addr);
	first_explored	= last_explored = addr;
	return true;
}

/*
 *
 */
void analyser::init_codeanalyser()
{
	code = new codeanalyser();
	code->init(this);
}

/*
 *
 */
void analyser::init_dataanalyser()
{
	data = new dataanalyser();
	data->init(this);
}

/*
 *
 */
void analyser::init_unasm()
{
	// abstract
}

/*
 *
 */
void	analyser::log(char *s)
{
	// stub
}

/*
 *
 */
CPU_ADDR analyser::mapaddr(ADDR Addr)
{
	/*
	 * 	this function should map the independent address Addr to a
	 * 	processor dependent
	 * 	e.g.    .23423 --> 00234324    (or something like this)
	 *   it is only used for relativ calls/jumps
	 */
	CPU_ADDR a;
	a.addr32=Addr;
	return a;
}

taddr *analysernewaddr(taddr **addrs, ADDR Addr)
{
	if (*addrs) {
		if (Addr < (*addrs)->addr) return analysernewaddr(&(*addrs)->left, Addr);
		if (Addr > (*addrs)->addr) return analysernewaddr(&(*addrs)->right, Addr);
	} else {
		*addrs = (taddr *) calloc(1, sizeof(taddr));
		(*addrs)->addr	= Addr;
	}
	return (*addrs);
}

/*
 *
 */
taddr *analyser::newaddr(ADDR	Addr)
{
	if (!(cur_addr_ops--)) {
		optimize_addr_tree();
	}
	return analysernewaddr(&addrs, Addr);
}

tlabel *analysernewlabel(tlabel **labels, char *label, taddr *Addr, labeltype type)
{
	if (*labels) {
		int i = strcmp(label, (*labels)->name);
		if (i < 0) return analysernewlabel(&((*labels)->left), label, Addr, type);
		if (i > 0) return analysernewlabel(&((*labels)->right), label, Addr, type);
		if (!((*labels)->addr)) goto w;
	} else {
		*labels = (tlabel *) calloc(1, sizeof(tlabel));
		(*labels)->name = label;
	    w:
		(*labels)->addr = Addr;
		(*labels)->type = type;
	}
	return (*labels);
}

/*
 *
 */
tlabel *analyser::newlabel(char *label, taddr *Addr, labeltype type, taddr *infunc)
{
	if (!(cur_label_ops--)) {
		optimize_label_tree();
	}
	tlabel *result = analysernewlabel(&labels, label, Addr, type);
	if ((result) && (result->addr==Addr) && (infunc)) setaddrfunc(Addr, infunc);
	return result;
}

/*
 *
 */
ADDR analyser::nextvalid(ADDR Addr)
{
	// abstract
	return 0;
}

/*
 *
 */
void	analyser::notifyprogress(ADDR	Addr)
{
	// stub
}

/*
 * optimizes(=balances) addr tree
 */
void analyser::optimize_addr_tree()
{
	cur_addr_ops = addrthreshold;
	// implement me!
}

/*
 * see optimize_addr_tree()
 */
void analyser::optimize_label_tree()
{
	cur_label_ops = labelthreshold;
	// implement me!
}

/*
 *
 */
bool analyser::popaddr(ADDR *Addr, ADDR *func)
{
	if (!addr_queue->empty()) {

		addrqueueitem *aqi = (addrqueueitem *) addr_queue->dequeue();
		*Addr = aqi->addr;
		*func = aqi->func;
		free(aqi);

		dprintf("addr %08lx (from sub %08lx) poped\n", *Addr, *func);
		return true;

	} else {
		dprintf("pop failed -> analyser obviously finished\n");
		return false;
	}
}

/*
 *
 */
void analyser::pushaddr(ADDR Addr, ADDR func)
{
	if (validcodeaddr(Addr))	{
		dprintf("addr %08lx (from func %08lx) pushed\n", Addr, func);
		addrqueueitem *aqi = (addrqueueitem *) malloc(sizeof(addrqueueitem));
		aqi->addr = Addr;
		aqi->func = func;
		addr_queue->enqueue((void *) aqi);
	}
}

void saveaddrs(ht_stream_filter *st, taddr *addr)
{
	if (addr) {
		saveaddrs(st, addr->left);

		st->put_int_hex(addr->addr, sizeof(ADDR), "addr");
		fstrput(st, addr->label->name);
		// xrefs
		txref	*xref = addr->xreflist;
		int		count = 0;
		while (xref) {
			count++;
			xref = xref->next;
		}
		st->put_int_dec(count, sizeof count, "count_of_xrefs");
		xref = addr->xreflist;
		while (xref) {
			st->put_int_hex(xref->addr, sizeof(ADDR), "xref_addr");
			st->put_int_hex(xref->type, sizeof(txreftype), "xreftype");
			xref = xref->next;
		}
		// comments
		tcomment	*com = addr->comments;
		count = 0;
		while (com) {
			count++;
			com = com->next;
		}
		st->put_int_dec(count, sizeof count, "count_of_comments");
		com = addr->comments;
		while (com) {
			fstrput(st, com->text);
			com = com->next;
		}
//		f->write(&addr->type, sizeof(taddrtype));

		saveaddrs(st, addr->right);
	}
}

/*
 *
 */
void analyser::store(ht_stream_filter *st)
{
	st->put_object(addr_queue, "addr_queue");
	st->put_int_dec(ops_parsed, sizeof ops_parsed, "ops_parsed");
	st->put_bool(active, "active");
	st->put_int_hex(next_explored, sizeof next_explored, "next_explored");
	st->put_int_hex(first_explored, sizeof first_explored, "first_explored");
	st->put_int_hex(last_explored, sizeof last_explored, "last_explored");
	st->put_int_hex(mode, sizeof mode, "mode");
	st->put_object(explored, "explored");
	st->put_int_dec(getaddrcount(), sizeof(int), "addr_count");
	saveaddrs(st, addrs);
	st->put_object(disasm, "disassembler");
	return;
}

/*
 *
 */
int	analyser::query_config(int mode)
{
	// stub
	return 0;
}

/*
 *
 */
bool	analyser::savetychangeaddr(ADDR Addr)
{
	addr	= Addr;
	return ((addr>=next_explored) || (!validcodeaddr(addr))) ? gotoaddr(addr, INVALID_ADDR) : true;
}

/*
 *
 */
void analyser::setactive(bool mode)
{
	if (mode) {
		if (!active) {
			active = true;
			some_analyser_active++;
		}
	} else {
		if (active) {
			active = false;
			some_analyser_active--;
		}
	}
}


/*
 *
 */
void analyser::setaddrfunc(taddr *a, taddr *func)
{
	if (a) {
		a->thisfunc = func;
		a->flags &= !AF_FUNCTION_END;
		a->flags |= AF_FUNCTION_SET;
	}
}

/*
 *
 */
void analyser::setdisasm(disassembler *Disasm)
{
	disasm = Disasm;
	if (disasm) maxopcodelength = disasm->getmaxopcodelength();
}

/*
 *	sets addr_threshold. after threshold addr_tree ops the tree will
 *   be optimized
 */
void analyser::set_addr_tree_optimize_threshold(int threshold)
{
	addrthreshold = threshold;
	if (cur_addr_ops > addrthreshold) cur_addr_ops = addrthreshold;
}

/*
 *	see set_addr_tree_optimize_threshold
 */
void analyser::set_label_tree_optimize_threshold(int threshold)
{
	labelthreshold = threshold;
	if (cur_label_ops > labelthreshold) cur_label_ops = labelthreshold;
}

/*
 *
 */
bool	analyser::validaddr(ADDR	Addr, tsectype	action)
{
	// abstract
	return 0;
}

/*
 *
 */
bool	analyser::validcodeaddr(ADDR Addr)
{
	taddr *a = findaddr(Addr);
	if (a) {
		if ((a->type.type != dtcode) && (a->type.type != dtunknown)) return false;
	}
	return validaddr(Addr, sccode);
}

/*
 *
 */
bool	analyser::validreadaddr(ADDR Addr)
{
	return validaddr(Addr, scread);
}

/*
 *
 */
bool	analyser::validwriteaddr(ADDR	Addr)
{
	return validaddr(Addr, scwrite);
}

/*
 *
 *		interface only (there's no internal use)
 *
 */

char bufbuf[1024];

char *analy_addr_sym_func(CPU_ADDR Addr, int *symstrlen)
{
	/* should only be called/used/assigned if theanaly is set */
	taddr *a = theanaly->findaddr(Addr.addr32);
	if ((a) && (a->label)){
		char *b=tag_make_ref(bufbuf, Addr.addr32, 0, a->label->name);
//		*b=0;
		if (symstrlen) *symstrlen=b-bufbuf;
		return bufbuf;
	} else {
		if (theanaly->validaddr(Addr.addr32, scvalid)) {
			char m[9];
			sprintf(m, "%xh", Addr.addr32);
			char *b=tag_make_ref(bufbuf, Addr.addr32, 0, m);
			*b=0;
			if (symstrlen) *symstrlen=b-bufbuf;
			return bufbuf;
		}
	}
	return 0;
}

/*
 *	should return a new instance of an apropriate assembler
 *
 */
assembler *analyser::create_assembler()
{
	return NULL;
}

/*
 *
 */
tcomment *analyser::getcomments(ADDR Addr)
{
	taddr *a = findaddr(Addr);
	if (a) {
		return a->comments;
	}
	return NULL;
}

/*
 *
 */
char	*analyser::getdisasmstr(ADDR Addr)
{
	if (validcodeaddr(Addr)) {
		if (disasm) {
			addr_sym_func = 0;
			OPCODE *o=disasm->decode((byte *)bufptr(Addr), maxopcodelength, mapaddr(Addr));
			return disasm->strf(o, X86DIS_STYLE_HEX_NOZEROPAD+X86DIS_STYLE_HEX_ASMSTYLE, DISASM_STRF_SMALL_FORMAT);
		} else {
			return "<no disassembler!>";
		}
	} else {
		return "";
	}
}

/*
 *
 */
char	*analyser::getdisasmstr_formatted(ADDR Addr)
{
	if (disasm) {
		theanaly = this;
		addr_sym_func = &analy_addr_sym_func;

		OPCODE *o=disasm->decode((byte *)bufptr(Addr), maxopcodelength, mapaddr(Addr));
		char *res = disasm->str(o, X86DIS_STYLE_HEX_NOZEROPAD+X86DIS_STYLE_HEX_ASMSTYLE);

		theanaly = NULL;
		addr_sym_func = 0;

		return res;
	} else {
		return "<no disassembler!>";
	}
}

char tempbuf[1024];
/*
 *
 */
bool	analyser::getformattedline(ADDR Addr, int line, char *buf, int *length)
{
	taddr *a = findaddr(Addr);
	int level	= 0;

	*length = 0;

	if (line==1) {
		char temp[9];
		sprintf(temp, HEX8FORMAT8, Addr);
		buf=tag_make_sel(buf, temp);
	} else {
		buf=tag_make_sel(buf, "   ...  ");
//		buf+=sprintf(buf, "   ...  ");
	}

	if (explored->contains(Addr))	{
		strcpy(buf, " ! ");
	} else {
		strcpy(buf, "   ");
	}
	buf+=3;

/*
	if (mode & ANALY_EDIT_BYTES) {
		// indent labels (if desired)
		j = strlen(buf);
		for (int i=0; i<=(maxopcodelength*2); i++) {
			buf[i+j]=' ';
		}
		buf[j+(maxopcodelength*2)+1]=0;
	}
 */

	if (a) {
		// comments
		tcomment *c = a->comments;
		while (c)	{
			level++;
			if (level==line) {
				strcpy(buf, c->text);
				return true;
			}
			c = c->next;
		}
		// label && xrefs
		bool collapsed_xrefs = false;
		if (mode & ANALY_COLLAPSE_XREFS) {
			// count xrefs
			int i=0;
			txref *x = a->xreflist;
			while (x && (i<21)) {
				x = x->next;
				i++;
			}
			if (i==21) {
				collapsed_xrefs = true;
			}
		}

		if ((a->label) || (a->xreflist)) {
			level ++;
			int labellength = (a->label) ? strlen(a->label->name) : 0;
			if (level==line) {
#define LABELINDENT 30
#define LABELINDENTSTR "30"
				int l;
				if (labellength) {
					l=sprintf(buf, "%-"LABELINDENTSTR"s", a->label->name);
					buf[labellength]=':';
				} else {
					l=sprintf(buf, "%"LABELINDENTSTR"s", " ");
				}
				buf+=l;
				*(buf+1)=0;
				// xrefs
				if (labellength>=LABELINDENT) return true;
#define MAX_XREFS 3
				if (collapsed_xrefs) {
					*buf++ = ' ';
					buf = tag_make_ref(buf, Addr, 1, "<< show xrefs >>");
				} else {
					txref *x = a->xreflist;
					if (x) {
						strcpy(buf, " ;xref");
						buf+=6;
					} else {
						*(buf-l+labellength+1)=0;
					}
					int i=0;
					while (x)	{
						if (i++<MAX_XREFS) {
							char buf3[9];
							sprintf(buf3, HEX8FORMAT, x->addr);
							*(buf++)=' ';
							*(buf++)=xref_type_short(x->type);
							buf=tag_make_ref(buf, x->addr, 0, buf3);
						} else return true;
						x = x->next;
					}
				}
				return true;
			}
			// xrefsend
			if ((labellength>=LABELINDENT) && (a->xreflist)) level++;
		}
		// !! evil, worse, devil, stupid, fucking code: !!
		if (!collapsed_xrefs) {
			txref *x = a->xreflist;
			int i=0, j=0;
			bool	onetimeshit= false;
			while (x)	{
				if (level==line) {

					if (!onetimeshit) {
						onetimeshit = true;
						strcpy(buf, "                               ;xref");
						buf+=strlen("                               ;xref");
					}
					char buf3[9];
					sprintf(buf3, HEX8FORMAT, x->addr);
					*(buf++)=' ';
					*(buf++)=xref_type_short(x->type);
					buf=tag_make_ref(buf, x->addr, 0, buf3);
					if ((j++)==MAX_XREFS) return true;
				}
				if ((((++i) % MAX_XREFS)==0) && (x->next)) level++;

				x = x->next;
			}
			if (onetimeshit) return true;
			// xrefsend
		} else {
			if (level==line) {
				strcpy(buf, "                               ");
				buf+=strlen("                               ");
				buf = tag_make_ref(buf, Addr, 1, "<< show xrefs >>");
				return true;
			}
		}
		// end
	}

	level ++;
	if (level==line) {
		//
		if (a) {
			if ((a->type.type == dtcode) || ((a->type.type == dtunknown) && (validcodeaddr(Addr)))) {
				// code
			} else {
				// data
			}
		} else {
			if (validcodeaddr(Addr)) {
			} else {
			}
		}
		//
		if (validcodeaddr(Addr))	{
			taddr *nextaddr = enum_addrs(Addr);
			int op_len;

			if (nextaddr) {
				op_len = MIN((dword)maxopcodelength, (nextaddr->addr - Addr));
			} else {
				op_len = maxopcodelength;
			}

			if (disasm) {
				OPCODE *o=disasm->decode((byte *)bufptr(Addr), op_len, mapaddr(Addr));
				/* inits for addr-sym transformations */
				theanaly = this;
				addr_sym_func = &analy_addr_sym_func;
				/**/
				char *x=disasm->str(o, X86DIS_STYLE_HEX_NOZEROPAD+X86DIS_STYLE_HEX_ASMSTYLE);
				char *t=tempbuf;
				int xl=tag_strlen(x);
				*(t++)=' ';
				*(t++)=' ';
				memmove(t, x, xl);
				t+=xl;
				*t=0;
				/* deinits for addr-sym transformations */
				theanaly = NULL;
				addr_sym_func = 0;
				/**/
				*length=disasm->getsize(o);
			} else {
				sprintf(tempbuf, "  db      0x%02x", *(byte *)bufptr(Addr));
				*length=1;
			}
		} else {
			if (validaddr(Addr, scvalid)) {
				if ((a) && (a->type.type == dtint)) {
					*length = a->type.length;
					assert(*length);
					byte c;
					if (validaddr(Addr, scinitialized)) {
						switch (a->type.intsubtype) {
							case dstiword:
								sprintf(tempbuf, "  dw      0x%04x", *(word *)bufptr(Addr));
								break;
							case dstidword:
								sprintf(tempbuf, "  dd      0x%08x", *(dword *)bufptr(Addr));
								break;
							case dstibyte:
							default:
								c = *(byte *)bufptr(Addr);
								sprintf(tempbuf, "  db      0x%02x ; `%c'", c, (c<32)?32:c);
						}
					} else {
						switch (a->type.intsubtype) {
							case dstiword:
								strcpy(tempbuf, "  dw      ????");
								break;
							case dstidword:
								strcpy(tempbuf, "  dd      ????????");
								break;
							case dstibyte:
							default:
								strcpy(tempbuf, "  db      ??");
						}
					}
				} else {
					*length =	1;
					if (validaddr(Addr, scinitialized)) {
						byte c = *(byte *)bufptr(Addr);
						sprintf(tempbuf, "  db      0x%02x ; `%c'", c, (c<32)?32:c);
					} else {
						strcpy(tempbuf, "  db      ??");
					}
				}
			} else {
				ADDR next = nextvalid(Addr);
				if (next != INVALID_ADDR) {
					*length = next - Addr;
					sprintf(tempbuf, "  db      ?? * %d", next - Addr);
				} else {
					*length =	1;
					strcpy(tempbuf, "  db      ??");
				}
			}
		}
		if (mode & ANALY_EDIT_BYTES) {
			if (validaddr(Addr, scinitialized)) {

				FILEADDR a=fileaddr(Addr);
				assert(a != INVALID_FILE_OFS);
				for (int i=0; i<(*length); i++) {
					buf = tag_make_edit_byte(buf, a+i);
				}
				for (int i=0; i<=(maxopcodelength*2)-(*length)*2; i++) {
					*(buf++)=' ';
				}
				*buf=0;
			}
		}
		int l=tag_strlen(tempbuf);
		memmove(buf, tempbuf, l);
		buf+=l;
		*buf=0;
		return true;
	}
	*buf=0;
	return false;
}

bool analyser::getformattedline_length(ADDR Addr, int line, int *length)
{
	taddr *a = findaddr(Addr);
	int level	= 0;

	*length = 0;

	if (a) {
		// comments
		tcomment *c = a->comments;
		while (c)	{
			level++;
			if (level==line) return true;
			c = c->next;
		}
		// label && xrefs
		bool collapsed_xrefs = false;
		if (mode & ANALY_COLLAPSE_XREFS) {
			// count xrefs
			int i=0;
			txref *x = a->xreflist;
			while (x && (i<21)) {
				x = x->next;
				i++;
			}
			if (i==21) {
				collapsed_xrefs = true;
			}
		}
		if (a->label) {
			level ++;
			int labellength = strlen(a->label->name);
			if (level==line) {
				if (labellength>=LABELINDENT) return true;
				// xrefs
				txref *x = a->xreflist;
				int i=0;
				while (x)	{
					if (i++<MAX_XREFS) {
					} else return true;
					x = x->next;
				}
				return true;
				// xrefsend
			}
			if ((labellength>=LABELINDENT) && (a->xreflist)) level++;
		}
		// !! evil, worse, devil, stupid, fucking code: !!
		if (!collapsed_xrefs) {
			txref *x = a->xreflist;
			int i=0,j=0;
			bool	onetimeshit= false;
			while (x)	{
				if (level==line) {
					onetimeshit = true;
					if ((j++)==MAX_XREFS) return true;
				}
				if ((((++i) % MAX_XREFS)==0) && (x->next)) level ++;
				x = x->next;
			}
			if (onetimeshit) return true;
			// xrefsend
		} else {
			if (level==line) return true;
		}
		// end
	}

	level ++;
	if (level==line) {
		if (validcodeaddr(Addr))	{
			taddr *nextaddr = enum_addrs(Addr);
			int op_len;
			if (nextaddr) {
				op_len = MIN((dword)maxopcodelength, (nextaddr->addr - Addr));
			} else {
				op_len = maxopcodelength;
			}
			if (disasm) {
				OPCODE *o=disasm->decode((byte *)bufptr(Addr), op_len, mapaddr(Addr));
				*length=disasm->getsize(o);
			} else {
				*length=1;
			}
		} else {
			if (validaddr(Addr, scvalid)) {
				if ((a) && (a->type.type == dtint)) {
					*length = a->type.length;
					assert(*length);
				} else {
					*length =	1;
				}
			} else {
				ADDR next = nextvalid(Addr);
				if (next != INVALID_ADDR) {
					*length = next - Addr;
				} else {
					*length =	1;
				}
			}
		}
		return true;
	}

	return false;
}

/*
 *
 */
char *analyser::getname()
{
	return "generic";
}

/*
 *
 */
txref *analyser::getxrefs(ADDR Addr)
{
	taddr *a = findaddr(Addr);
	if (!a) return NULL;
	return a->xreflist;
}

/*
 *
 */
void analyser::setdisplaymode(int enable, int disable)
{
	mode |= enable;
	mode &= ~disable;
}

/*
 *
 */
void analyser::toggledisplaymode(int toggle)
{
	mode ^= toggle;
}

/*
 *	converts FILEADDR fileaddr to ADDR
 */
ADDR analyser::vaddr(FILEADDR fileaddr)
{
	// abstract
	return INVALID_ADDR;
}

/****************************************************************************/

/*
 *
 */
void analy_disassembler::init(analyser *A)
{
	analy = A;
	disasm = NULL;
	init_disasm();
}

/*
 *
 */
int  analy_disassembler::load(ht_stream_filter *f)
{
	return 0;
}

/*
 *
 */
void analy_disassembler::done()
{
}

/*
 *
 */
ADDR analy_disassembler::branchaddr(OPCODE *opcode)
{
	return INVALID_ADDR;
}

/*
 *
 */
void analy_disassembler::init_disasm()
{
	if (analy) {
		analy->setdisasm(disasm);
	}
}

/*
 *
 */
tbranchtype analy_disassembler::isbranch(OPCODE *opcode)
{
	return brnobranch;
}

/*
 *
 */
void analy_disassembler::store(ht_stream_filter *f)
{
}

