/*
 *	area.cc
 */

/*	Copyright (C) 1999, 2000, 2001 Sebastian Biallas (sb@web-productions.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <stdlib.h>
#include "area.h"
#include "store.h"

void area::init()
{
	a = NULL;
}

void areaload(ht_stream_filter *f, area_s **p, int level, int *left)
{
	if ((*left)<=0) {
		*p = NULL;
		return;
	}
	*p = (area_s *) calloc(sizeof(area_s), 1);
	if ((level==1) || (*left==1)) {
		(*p)->start = f->get_int_hex(sizeof (dword), "start");
		(*p)->end = f->get_int_hex(sizeof (dword), "end");
		(*left)--;
	} else {
		areaload(f, &(*p)->left, level / 2, left);
		(*p)->start = f->get_int_hex(sizeof (dword), "start");
		(*p)->end = f->get_int_hex(sizeof (dword), "end");
		(*left)--;
		areaload(f, &(*p)->right, level / 2, left);
	}
}

int	area::load(ht_stream_filter *f)
{
	int count;
	count = f->get_int_dec(sizeof count, "count");
	areaload(f, &a, count, &count);
	return 0;
}

void area::done()
{
	free_recursive(a);
}

void areaadd(area_s **p, dword Start, dword End)
{
	if (*p) {
		if ((Start >= (*p)->start) && (Start <= (*p)->end)) {
			if ((*p)->end < End) (*p)->end = End;
			if ((End >= (*p)->start) && (End <= (*p)->end)) {
				if ((*p)->start > Start) (*p)->start = Start;
			}
			return;
		}
		if ((End >= (*p)->start) && (End <= (*p)->end)) {
			if ((*p)->start > Start) (*p)->start = Start;
			return;
		}
		if (Start > (*p)->end) areaadd(&(*p)->right, Start, End);
					  else  areaadd(&(*p)->left, Start, End);
	} else {
		// new p
		area_s	*tmp = (area_s *) malloc(sizeof(area_s));
		*p = tmp;
		(*p)->start = Start;
		(*p)->end = End;
		(*p)->left = NULL;
		(*p)->right = NULL;
	}
}

void area::add(dword Start, dword End)
{
	areaadd(&a, Start, End);
}

bool areacontains(area_s *p, dword V)
{
	if (p) {
		if (V < (p->start)) return areacontains(p->left, V);
		if (V >= (p->end)) return areacontains(p->right, V);
		/*if ((V >= (p->start)) && (V < (p->end)))*/ return true;
	} else return false;
}

bool area::contains(dword V)
{
	return areacontains(a, V);
}

void areafindnext(area_s *p, dword From, dword *res)
{
	if (From < p->start) {
		*res = p->start;
		if (p->left) areafindnext(p->left, From, res);
	} else if (From >= p->end) {
		if (p->right) areafindnext(p->right, From, res);
	} else *res = From;
}

dword  area::findnext(dword From)
{
	dword res = 0xffffffff;
	if (a) areafindnext(a, From, &res);
	return res;
}

void areafindprev(area_s *p, dword From, dword *res)
{
//FIXME ??:
	if (p) {
		if (From < p->start) {
			if (p->left) areafindprev(p->left, From, res);
		} else if (From >= p->end) {
			*res = p->start;
			areafindprev(p->right, From, res);
		} else *res = From;
	} else *res = 0x0;
}

dword area::findprev(dword From)
{
	dword res;
	areafindprev(a, From, &res);
	return res;
}

void area::free_recursive(area_s *p)
{
	if (p) {
		free_recursive(p->left);
		free_recursive(p->right);
		free(p);
	}
}

void areacount(area_s *p, int *c)
{
	if (p) {
		areacount(p->left, c);
		(*c)++;
		areacount(p->right, c);
	}
}

void areastore(ht_stream_filter *f, area_s *p)
{
	if (p) {
		areastore(f, p->left);
		f->put_int_hex(p->start, sizeof(dword), "start");
		f->put_int_hex(p->end, sizeof(dword), "end");
		areastore(f, p->right);
	}
}

void area::store(ht_stream_filter *f)
{
	int count = 0;
	areacount(a, &count);
	f->put_int_dec(count, sizeof count, "count");
	areastore(f, a);
}

#ifdef DEBUG
void areadump(int sp, area_s *A)
{
	if (A) {
		for (int i=0; i<sp; i++) printf(" ");
		printf("%08x %08x\n", A->start, A->end);
		++sp;++sp;
		areadump(sp, A->left);
		areadump(sp, A->right);
	}
}

void area::dump()
{
	areadump(1, a);
}
#endif
