/* 
 *	HT Editor
 *	htelfhd.cc
 *
 *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "elfstruc.h"
#include "htatom.h"
#include "htelf.h"
#include "htelfhd.h"
#include "httag.h"
#include "formats.h"

hts_mask_ptable elfheader[]=
{
	{"ident", 0},
	{"    magic",		STATICTAG_EDIT_BYTE("00000000")" "STATICTAG_EDIT_BYTE("00000001")" "STATICTAG_EDIT_BYTE("00000002")" "STATICTAG_EDIT_BYTE("00000003")" = "STATICTAG_EDIT_CHAR("00000000") STATICTAG_EDIT_CHAR("00000001") STATICTAG_EDIT_CHAR("00000002") STATICTAG_EDIT_CHAR("00000003")},
	{"    class",		STATICTAG_EDIT_BYTE("00000004")" ("STATICTAG_DESC_BYTE("00000004", ATOM_ELF_CLASS_STR)")"},
	{"    data",		STATICTAG_EDIT_BYTE("00000005")" ("STATICTAG_DESC_BYTE("00000005", ATOM_ELF_DATA_STR)")"},
	{"    version",	STATICTAG_EDIT_BYTE("00000006")},
	{"    reserved",	STATICTAG_EDIT_BYTE("00000007")" "STATICTAG_EDIT_BYTE("00000008")" "STATICTAG_EDIT_BYTE("00000009")" "STATICTAG_EDIT_BYTE("0000000a")" "STATICTAG_EDIT_BYTE("0000000b")" "STATICTAG_EDIT_BYTE("0000000c")" "STATICTAG_EDIT_BYTE("0000000d")" "STATICTAG_EDIT_BYTE("0000000e")" "STATICTAG_EDIT_BYTE("0000000f")},
	{0, 0}
};
	
hts_mask_ptable elfheader32lsb[]=
{
	{"type",					STATICTAG_EDIT_WORD("00000010")" ("STATICTAG_DESC_WORD("00000010", ATOM_ELF_TYPE_STR)")"},
	{"machine",				STATICTAG_EDIT_WORD("00000012")" ("STATICTAG_DESC_WORD("00000012", ATOM_ELF_MACHINE_STR)")"},
	{"version",				STATICTAG_EDIT_DWORD("00000014")},
	{"entrypoint",				STATICTAG_EDIT_DWORD("00000018")},
	{"program header offset",	STATICTAG_EDIT_DWORD("0000001c")},
	{"section header offset",	STATICTAG_EDIT_DWORD("00000020")},
	{"flags",					STATICTAG_EDIT_DWORD("00000024")},
	{"elf header size",			STATICTAG_EDIT_WORD("00000028")},
	{"program header entry size",	STATICTAG_EDIT_WORD("0000002a")},
	{"program header count",		STATICTAG_EDIT_WORD("0000002c")},
	{"section header entry size",	STATICTAG_EDIT_WORD("0000002e")},
	{"section header count",		STATICTAG_EDIT_WORD("00000030")},
	{"section header strtab section index",STATICTAG_EDIT_WORD("00000032")},
	{0, 0}
};

int_hash elf_class[] =
{
	{ELFCLASSNONE, "invalid class type"},
	{ELFCLASS32, "32-bit objects"},
	{ELFCLASS64, "64-bit objects"},
	{0, 0}
};

int_hash elf_data[] =
{
	{ELFDATANONE, "invalid data encoding"},
	{ELFDATA2LSB, "LSB encoding"},
	{ELFDATA2MSB, "MSB encoding"},
	{0, 0}
};

int_hash elf_type[] =
{
	{ELF_ET_NONE, "no file type"},
	{ELF_ET_REL, "relocatable file"},
	{ELF_ET_EXEC, "executable file"},
	{ELF_ET_DYN, "shared object file"},
	{ELF_ET_CORE, "core file"},
	{0, 0}
};

int_hash elf_machine[] =
{
	{ELF_EM_NONE, "no machine"},
	{ELF_EM_M32, "AT&T WE32100"},
	{ELF_EM_SPARC, "SPARC"},
	{ELF_EM_386, "Intel 80386"},
	{ELF_EM_68K, "Motorola 68000"},
	{ELF_EM_88K, "Motorola 88000"},
	{ELF_EM_860, "Intel 80860"},
	{ELF_EM_MIPS, "MIPS RS3000"},
	{0x32, "Intel IA-64 ???"},
	{0, 0}
};

ht_view *htelfheader_init(bounds *b, ht_stream *file, ht_format_group *group)
{
	ht_elf_shared_data *elf_shared=(ht_elf_shared_data *)group->get_shared_data();
	
	ht_uformat_view *v=new ht_uformat_view();
	v->init(b, DESC_ELF_HEADER, VC_EDIT, file, group);
	hts_mask *m=new hts_mask();
	m->init(file, 0);
	char info[128];
	sprintf(info, "* ELF header at offset %08x", elf_shared->header_ofs);
	register_atom(ATOM_ELF_CLASS, elf_class);
	register_atom(ATOM_ELF_DATA, elf_data);
	register_atom(ATOM_ELF_TYPE, elf_type);
	register_atom(ATOM_ELF_MACHINE, elf_machine);
	m->add_mask(info);
	m->add_staticmask_ptable(elfheader, elf_shared->header_ofs);
	
	switch (elf_shared->header.e_ident[ELF_EI_CLASS]) {
		case ELFCLASS32:
			switch (elf_shared->header.e_ident[ELF_EI_DATA]) {
				case ELFDATA2LSB:
					m->add_staticmask_ptable(elfheader32lsb, elf_shared->header_ofs);
					break;
			}
			break;
	}
	
	v->insertsub(m);
	return v;
}

format_view_if htelfheader_if = {
	htelfheader_init,
	0
};
