/* 
 *	HT Editor
 *	htelfsym.cc
 *
 *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "elfstruc.h"
#include "htatom.h"
#include "htelf.h"
#include "htelfsym.h"
#include "httag.h"
#include "formats.h"

int_hash elf_st_bind[] =
{
	{ELF_STB_LOCAL, 	"local"},
	{ELF_STB_GLOBAL, 	"global"},
	{ELF_STB_WEAK, 	"weak"},
	{0, 0}
};

int_hash elf_st_type[] =
{
	{ELF_STT_NOTYPE,	"no type"},
	{ELF_STT_OBJECT,	"object"},
	{ELF_STT_FUNC,		"func"},
	{ELF_STT_SECTION,	"section"},
	{ELF_STT_FILE,		"file"},
	{0, 0}
};

ht_view *htelfsymboltable_init(bounds *b, ht_stream *file, ht_format_group *group)
{
	ht_elf_shared_data *elf_shared=(ht_elf_shared_data *)group->get_shared_data();

	if ((elf_shared->header.e_ident[ELF_EI_CLASS]!=ELFCLASS32) || 
	(elf_shared->header.e_ident[ELF_EI_DATA]!=ELFDATA2LSB)) return 0;
	
	UINT skip=elf_shared->symtables;
	UINT symtab_shidx=ELF_SHT_NULL;
	for (UINT i=1; i<elf_shared->sheaders.count; i++) {
		if ((elf_shared->sheaders.sheaders[i].sh_type==ELF_SHT_SYMTAB) || (elf_shared->sheaders.sheaders[i].sh_type==ELF_SHT_DYNSYM)) {
			if (!skip--) {
				symtab_shidx=i;
				break;
			}
		}
	}
	if (symtab_shidx==ELF_SHT_NULL) return 0;

	STREAMOFS h=elf_shared->sheaders.sheaders[symtab_shidx].sh_offset;
	STREAMOFS sto=elf_shared->sheaders.sheaders[elf_shared->sheaders.sheaders[symtab_shidx].sh_link].sh_offset;

	file->seek(elf_shared->sheaders.sheaders[elf_shared->header.e_shstrndx].sh_offset+elf_shared->sheaders.sheaders[symtab_shidx].sh_name);
	char *symtab_name=getstrz(file);
	char desc[128];
	sprintf(desc, DESC_ELF_SYMTAB, symtab_name, symtab_shidx);
	delete symtab_name;
	
	ht_uformat_view *v=new ht_uformat_view();
	v->init(b, desc, VC_EDIT, file, group);
	
	hts_mask *m=new hts_mask();
	m->init(file, 0);
	
	register_atom(ATOM_ELF_ST_BIND, elf_st_bind);

	char t[1024];	/* FIXME: possible buffer overflow ! */
	sprintf(t, "* ELF symtab at offset %08x", h);
	
	m->add_mask(t);

	UINT symnum=elf_shared->sheaders.sheaders[symtab_shidx].sh_size / sizeof (ELF_SYMBOL);
	for (UINT i=0; i<symnum; i++) {
		ELF_SYMBOL sym;
		file->seek(h+i*sizeof (ELF_SYMBOL));
		file->read(&sym, sizeof sym);
		file->seek(sto+sym.st_name);
		char *name=getstrz(file);


		char *tt=t;

		char *bind=matchhash(ELF32_ST_BIND(sym.st_info), elf_st_bind);
		if (bind) {
			tt+=sprintf(tt, "%-8s ", bind);
		} else {
			tt+=sprintf(tt, "? (%d) ", ELF32_ST_BIND(sym.st_info));
		}

		char *type=matchhash(ELF32_ST_TYPE(sym.st_info), elf_st_type);
		if (type) {
			tt+=sprintf(tt, "%-8s ", type);
		} else {
			tt+=sprintf(tt, "? (%d) ", ELF32_ST_TYPE(sym.st_info));
		}

		
		STREAMOFS so=elf_shared->sheaders.sheaders[elf_shared->header.e_shstrndx].sh_offset;
		tt+=sprintf(tt, "%08x %08x ", sym.st_value, sym.st_size);
		switch (sym.st_shndx) {
			case ELF_SHN_UNDEF:
				tt+=sprintf(tt, "*undefined  ");
				break;
			case ELF_SHN_ABS:
				tt+=sprintf(tt, "*absolute   ");
				break;
			case ELF_SHN_COMMON:
				tt+=sprintf(tt, "*common     ");
				break;
			default: {
				file->seek(so+elf_shared->sheaders.sheaders[sym.st_shndx].sh_name);
				char *s=getstrz(file);
				tt+=sprintf(tt, "%-11s ", s);
				delete s;
				break;
			}
		}
		tt+=sprintf(tt, "%s", name);
		delete name;
		m->add_mask(t);
	}

	v->insertsub(m);
	return v;
}

format_view_if htelfsymboltable_if = {
	htelfsymboltable_init,
	0
};
