/*
 *	HT Editor
 *	hthelp.cc
 *
 *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "htapp.h"
#include "htctrl.h"
#include "hthelp.h"
#include "htinfo.h"
#include "htiobox.h"
#include "htkeyb.h"
#include "htpal.h"
#include "htstring.h"
#include "tools.h"

#include <string.h>
#include <stdlib.h>

#include "hthelp.inc"

ht_help *help;

#define palclasskey_help	     				"help"

#define palkey_help_default					"help"

#define palcolors_help						3

#define palidx_help_normal					0
#define palidx_help_link_selected				1
#define palidx_help_link_unselected			2

/*
 *	Info files
 */

#define INFO_MAX_LINE_LENGTH 1024

char readbuf[INFO_MAX_LINE_LENGTH];	/* FIXME: possible buffer overflow ! */

void cleanup_str(char *s)
{
	while (*s) {
		if ((unsigned char)*s<32) *s=32;
		s++;
	}
}

int read_whitespaces(ht_stream *file, char *c)
{
	while (*c<=32) {
		if (!file->read(c, 1)) return 0;
	}
	return 1;
}

int read_skip(ht_stream *file, char *c)
{
	return file->read(c, 1);
}

char *read_until(ht_stream *file, char *c, char *stop)
{
	char *s=readbuf;
	while (!strchr(stop, *c)) {
		*(s++)=*c;
		if (!file->read(c, 1)) return 0;
		if (s-readbuf>=INFO_MAX_LINE_LENGTH) break;
	}
	*s=0;
	return readbuf;
}

int read_skip_until(ht_stream *file, char *c, char *stop)
{
	while (!strchr(stop, *c)) {
		if (!file->read(c, 1)) return 0;
	}
	return 1;
}

int read_info_file_node(ht_stream *file, char *c)
{
	char *x;
	if (!read_skip(file, c)) return 0;
	if (!read_whitespaces(file, c)) return 0;
	char *filename=ht_strdup(read_until(file, c, ","));
	if ((!read_skip(file, c)) || (!read_whitespaces(file, c))) {
		delete filename;
		return 0;
	}		
	x=read_until(file, c, ":");
	if ((!x) || (strcmp(x, "Node")!=0) || (!read_skip(file, c)) || (!read_whitespaces(file, c))) {
		delete filename;
		return 0;
	}		
	char *nodename=ht_strdup(read_until(file, c, ","));
	x=read_until(file, c, "\n");
	if ((!x) || (!read_skip(file, c))) {
		   delete filename;
		delete nodename;
		return 0;
	}	    

//     printf("file: %s, node: %s\n", filename, nodename);

/* file already there ? */
	ht_helpfile *hf=help->findfile(filename);
	if (!hf) {
		hf=new ht_helpfile();
		hf->init(filename);
		help->insertfile(hf);
	}

/* duplicate node ? */
	if (hf->findnode(nodename)) {
		   delete filename;
		delete nodename;
		return 0;
	}	    

/* create the node */
	ht_helpnode *hn=new ht_helpnode();
	hn->init(nodename);
	   delete filename;
	delete nodename;
	hf->insertnode(hn);

/* read the content line by line */
	while (1) {
		if ((*c==0) || (*c==0x1f)) {
			break;
		}
		x=read_until(file, c, "\n");
		if (!x) return 0;
		if (!read_skip(file, c)) return 0;
		cleanup_str(x);
		hn->content->insert_string(x);
	}

	return 1;
}

int read_info_file_tags(ht_stream *file, char *c)
{
	read_skip_until(file, c, "\n");
	read_skip(file, c);
/* read tags */
	/* FIXME: do we need the tag table ? */
	read_skip_until(file, c, "\x1f");
	return 1;
}

int read_info_file_block_by_name(char *fname, ht_stream *parent);

int read_info_file_indirect(ht_stream *file, char *c)
{
	if (!read_skip_until(file, c, "\n")) return 0;
	if (!read_skip(file, c)) return 0;

	while (1) {
		char *x=read_until(file, c, ":");
		if (!x) break;
		read_info_file_block_by_name(x, file);
		if (!read_until(file, c, "\n")) return 0;
		if (!read_skip(file, c)) return 0;
	}

	return 1;
}

int read_info_file_block(ht_stream *file, char *c)
{
	read_whitespaces(file, c);
	char *blocktype=read_until(file, c, ":");
	if (strcmp(blocktype, "File")==0) {
		if (!read_info_file_node(file, c)) return 0;
	} else if (strcmp(blocktype, "Tag Table")==0) {
		if (!read_info_file_tags(file, c)) return 0;
	} else if (strcmp(blocktype, "Indirect")==0) {
		if (!read_info_file_indirect(file, c)) return 0;
	} else {
//     	printf("unknown blocktype: %s\n", blocktype);
	}
	return 1;
}

int read_info_file_block_by_name(char *fname, ht_stream *parent)
{
/* FIXME: dirty code: */
	char *n=parent->get_name();
	char *l=strrchr(n, '/');
	char *p=0;
	if (l) {
		int e=strlen(fname);
		p=(char*)malloc(l-n+2+e);
		memmove(p, n, l-n+1);
		memmove(p+(l-n)+1, fname, e);
		p[l-n+e+1]=0;
	} else {
		p=ht_strdup(fname);
	}
/**/
	int r=0;
	ht_file_stream *f=new ht_file_stream();
	f->init(p, SAM_READ);
	if (!f->get_error()) r=read_info_file(f);
	delete p;
	delete f;
	return r;
}

int read_info_file(ht_stream *file)
{
	char c=0;
	file->seek(0);
	file->read(&c, 1);
	read_skip_until(file, &c, "\x1f");
	while (c==0x1f) {
		if (!read_info_file_block(file, &c)) return 0;
	}
	return 1;
}

/*
 *	CLASS ht_helpnode
 */

void ht_helpnode::init(char *_name)
{
	ht_data::init();
	name=ht_strdup(_name);
	content=new ht_string_list();
	content->init();
}

void ht_helpnode::done()
{
	delete name;
	content->destroy();
	delete content;
	ht_data::done();
}

void ht_helpnode::debug_dump()
{
	printf("node: %s\n", name);
}

/*
 *	CLASS ht_helpfile
 */

void ht_helpfile::init(char *_name)
{
	ht_data::init();
	name=ht_strdup(_name);
	nodes=new ht_clist();
	((ht_clist*)nodes)->init();
}

void ht_helpfile::done()
{
	delete name;
	nodes->destroy();
	delete nodes;
	ht_data::done();
}

ht_helpnode *ht_helpfile::findnode(char *name)
{
	int c=nodes->count();
	ht_helpnode *n;
	for (int i=0; i<c; i++) {
		n=(ht_helpnode*)nodes->get(i);
		if (strcmp(n->name, name)==0) return n;
	}
	return 0;
}

void ht_helpfile::insertnode(ht_helpnode *node)
{
	nodes->insert(node);
}

void ht_helpfile::debug_dump()
{
	printf("file %s\n", name);
	int c=nodes->count();
	ht_helpnode *n;
	for (int i=0; i<c; i++) {
		n=(ht_helpnode*)nodes->get(i);
		n->debug_dump();
	}
}

/*
 *	CLASS ht_help
 */

void ht_help::init()
{
	object::init();
	files=new ht_clist();
	((ht_clist*)files)->init();
}

void ht_help::done()
{
	files->destroy();
	delete files;
	object::done();
}

ht_helpfile *ht_help::findfile(char *name)
{
	int c=files->count();
	ht_helpfile *n;
	for (int i=0; i<c; i++) {
		n=(ht_helpfile*)files->get(i);
		if (strcmp(n->name, name)==0) return n;
	}
	return 0;
}

void ht_help::insertfile(ht_helpfile *file)
{
	files->insert(file);
}

void ht_help::debug_dump()
{
	printf("help dump\n");
	int c=files->count();
	ht_helpfile *n;
	for (int i=0; i<c; i++) {
		n=(ht_helpfile*)files->get(i);
		n->debug_dump();
	}
}

/*
 *	CLASS ht_helpviewer_status
 */

void ht_helpviewer_status::init()
{
}

void ht_helpviewer_status::done()
{
	if (filename) delete filename;
	if (nodename) delete nodename;
}

/*
 *	CLASS ht_helpviewer
 */

void ht_helpviewer::init(bounds *b, ht_scrollbar *_scrollbar)
{
	ht_viewer::init(b, "help", 0);
	VIEW_DEBUG_NAME("ht_helpviewer");

	scrollbar=_scrollbar;
	content=0;
	filename=0;
	nodename=0;
	cursorx=0;
	cursory=0;
	ofsx=0;
	ofsy=0;
	history=new ht_clist();
	((ht_clist*)history)->init();
}

void ht_helpviewer::done()
{
	history->destroy();
	delete history;
	ht_viewer::done();
}

int ht_helpviewer::cursor_up(int n)
{
	cursory-=n;
	if (cursory<0) {
		scroll_up(-cursory);
		cursory=0;
	}
	update_scrollbar();
	return n;
}

int ht_helpviewer::cursor_down(int n)
{
	int c=content->count();
	cursory+=n;
	if (cursory>c-1-ofsy) {
		cursory=c-1-ofsy;
	}
	if (cursory>size.h-1) {
		scroll_down(cursory-size.h+1);
		cursory=size.h-1;
	}
	if (cursory>c-1-ofsy) {
		cursory=c-1-ofsy;
	}
	update_scrollbar();
	return n;
}

int ht_helpviewer::cursor_left(int n)
{
	cursorx-=n;
	if (cursorx<0) {
		scroll_left(-cursorx);
		cursorx=0;
	}
	return n;
}

int ht_helpviewer::cursor_right(int n)
{
	cursorx+=n;
	if (cursorx>size.w-1) {
		scroll_right(cursorx-size.w+1);
		cursorx=size.w-1;
	}
	return n;
}

void ht_helpviewer::cursor_home()
{
	ofsx=0;
	cursorx=0;
}

void ht_helpviewer::cursor_end()
{
	if (content) {
		cursorx=size.w-4;
		ofsx=curline_vlen-size.w+4;
		if (ofsx<0) {
			cursorx=curline_vlen;
			ofsx=0;
		}
	}
}

char *ht_helpviewer::defaultpalette()
{
	return palkey_help_default;
}

char *ht_helpviewer::defaultpaletteclass()
{
	return palclasskey_help;
}

void ht_helpviewer::draw()
{
	curlink_node_dest=0;
	curlink_file_dest=0;
	clear(getcolor(palidx_help_normal));
	if (content) {
		int c=content->count()-ofsy;
		int d=c;
		if (d>size.h) d=size.h;
		for (int i=0; i<d; i++) {
			char *s=content->get_string(i+ofsy);
			help_print(0, i, s);
		}
//		printf(0, 0, help_color_normal, "F:%s, N:%s, Df:%s, Dn:%s", filename, nodename, curlink_file_dest, curlink_node_dest);
	}
	if (focused) setcursor(cursorx, cursory);
}

int ht_helpviewer::gotonode(char *file, char *node)
{
	ht_helpfile *hf=help->findfile(file);
	if (!hf) return 0;
	ht_helpnode *hn=hf->findnode(node);
	if (!hn) return 0;

	if ((filename) && (nodename)) {
		ht_helpviewer_status *status=new ht_helpviewer_status();
		status->cursorx=cursorx;
		status->cursory=cursory;
		status->ofsx=ofsx;
		status->ofsy=ofsy;
		status->filename=ht_strdup(filename);
		status->nodename=ht_strdup(nodename);
		history->insert(status);
	}		

	content=hn->content;
	if (filename) delete filename;
	filename=ht_strdup(file);
	if (nodename) delete nodename;
	nodename=ht_strdup(node);
	ofsx=0;
	ofsy=0;
	cursorx=0;
	cursory=0;
	return 1;
}

void ht_helpviewer::handlemsg(htmsg *msg)
{
	if (msg->msg==msg_keypressed) {
		switch (msg->data1.integer) {
			case K_Up:
				cursor_up(1);
				dirtyview();
				clearmsg(msg);
				return;
			case K_Down:
				cursor_down(1);
				dirtyview();
				clearmsg(msg);
				return;
			case K_Left:
				cursor_left(1);
				dirtyview();
				clearmsg(msg);
				return;
			case K_Right:
				cursor_right(1);
				dirtyview();
				clearmsg(msg);
				return;
			case K_Home:
				cursor_home();
				dirtyview();
				clearmsg(msg);
				return;
			case K_End:
				cursor_end();
				dirtyview();
				clearmsg(msg);
				return;
			case K_PageUp:
				scroll_up(size.h);
				dirtyview();
				clearmsg(msg);
				return;
			case K_PageDown:
				scroll_down(size.h);
				dirtyview();
				clearmsg(msg);
				return;
/*          	case K_Control_Left:
				scroll_left(2);
				dirtyview();
				clearmsg(msg);
				return;
			case K_Control_Right:
				scroll_right(2);
				dirtyview();
				clearmsg(msg);
				return;*/
			case K_BackSpace:
			case K_Alt_F1:
				reversenode();
				update_scrollbar();
				dirtyview();
				clearmsg(msg);
				return;
			case K_Space:
			case K_Return:
				if (curlink_node_dest && curlink_file_dest) gotonode(curlink_file_dest, curlink_node_dest);
				update_scrollbar();
				dirtyview();
				clearmsg(msg);
				return;
			case K_Control_PageUp:
				ofsy=0;
				cursorx=0;
				cursory=0;
				update_scrollbar();
				dirtyview();
				clearmsg(msg);
				return;
			case K_Control_PageDown: {
				int c=content->count();
				int vy=MIN(size.h-1, c-1);
				ofsy=c-vy-1;
				cursorx=0;
				cursory=vy;
				update_scrollbar();
				dirtyview();
				clearmsg(msg);
				return;
			}
			case K_Escape: {
				htmsg m;
				m.msg=cmd_window_close;
				((ht_app*)app)->queuemsg(app, &m);
				clearmsg(msg);
				return;
			}
		}
	}
	ht_viewer::handlemsg(msg);
}

void ht_helpviewer::help_print(int x, int y, char *str)
{
	int l=strlen(str);
	int a=0;
	if (l>ofsx) {
		char *s=str, *t;
		while (1) {
			t=strchr(s, '*');
			if (!t) break;
			if (x+a<ofsx) {
				if (t-s>ofsx) {
					lprint(0, y, getcolor(palidx_help_normal), MIN(size.w-a, t-s-ofsx), s+ofsx);
				}
			} else {
				lprint(x+a-ofsx, y, getcolor(palidx_help_normal), MIN(size.w-a, t-s), s);
			}
			a+=t-s;
			char *v, *w;
			if (strncmp(t, "*Note ", 6)==0) {
				v=t+6;
				w=strchr(v, ':');
			} else if ((t==str) && (strncmp(t, "* ", 2)==0)) {
				v=t+2;
				w=strchr(v, ':');
			} else {
				s=t+1;
				continue;
			}
			if ((w) && ((strchr(w+1, '.') || strchr(w+1, ',') || strchr(w+1, ':')))) {
				int cursor_here=((y==cursory) && (cursorx>=a) && (cursorx<a+w-v));
				int col=cursor_here ? getcolor(palidx_help_link_selected) :
					getcolor(palidx_help_link_unselected);
				if (x+a<ofsx) {
					if (w-v>ofsx) {
						lprint(0, y, col, MIN(size.w-a, w-v-ofsx), v+ofsx);
					}
				} else {
					lprint(x+a-ofsx, y, col, MIN(size.w-a, w-v), v);
				}
				a+=w-v;
				s=w+1;
				char *p=0, *d=curlink_node_dest_s;
				int start=1;
				while (!p) {
					p=strchr(":,.", *s);
					if (!*s) break;
					if (cursor_here) {
						if ((*s>32) || (!start)) {
							*(d++)=*s;
							start=0;
						}
					}
					s++;
				}
				if (cursor_here) {
					if (*p!=':') {
						*(d-1)=0;
						curlink_node_dest=curlink_node_dest_s;
						strcpy(curlink_file_dest_s, filename);
						curlink_file_dest=curlink_file_dest_s;
					} else {
						strncpy(curlink_node_dest_s, v, w-v);
						curlink_node_dest_s[w-v]=0;
						curlink_node_dest=curlink_node_dest_s;
						strcpy(curlink_file_dest_s, filename);
						curlink_file_dest=curlink_file_dest_s;
					}
				}
			} else {
				s=t+1;
			}
		}
		int l=strlen(s);
		if (x+a<ofsx) {
			if (l>ofsx) {
				lprint(0, y, getcolor(palidx_help_normal), size.w-a, s+ofsx);
				a+=l-ofsx;
			}
		} else {
			lprint(x+a-ofsx, y, getcolor(palidx_help_normal), size.w-a, s);
			a+=l;
		}
	}
	if (cursory==y) {
		curline_vlen=a+ofsx;
	}
}

void ht_helpviewer::releasefocus()
{
	ht_viewer::releasefocus();
	hidecursor();
}

void ht_helpviewer::reversenode()
{
	int c=history->count();
	if (!c) return;
	ht_helpviewer_status *status=(ht_helpviewer_status*)history->get(c-1);

	ht_helpfile *hf=help->findfile(status->filename);
	if (!hf) return;

	ht_helpnode *hn=hf->findnode(status->nodename);
	if (!hn) return;

	if (filename) delete filename;
	filename=ht_strdup(status->filename);

	if (nodename) delete nodename;
	nodename=ht_strdup(status->nodename);

	ofsx=status->ofsx;
	ofsy=status->ofsy;
	cursorx=status->cursorx;
	cursory=status->cursory;

	content=hn->content;

	history->del(c-1);
}

void ht_helpviewer::scroll_up(int n)
{
	ofsy-=n;
	if (ofsy<0) ofsy=0;
}

void ht_helpviewer::scroll_down(int n)
{
	int c=content->count();
	int vy=MIN(size.h-1, c-1);
	ofsy+=n;
	if (ofsy+vy>c) {
		ofsy=c-vy;
	}
}

void ht_helpviewer::scroll_left(int n)
{
	ofsx-=n;
	if (ofsx<0) {
		ofsx=0;
	}
}

void ht_helpviewer::scroll_right(int n)
{
	ofsx+=n;
}

void ht_helpviewer::update_scrollbar()
{
	if (content) {
		int c=content->count();
		if (c>1) scrollbar->setpos((ofsy+cursory)*100/(c-1));
	}
}

ht_mem_stream *htinfostream;

/* HELP */

defpal defpal_help[palcolors_help]={
{palidx_help_normal, "normal", VCP(VC_BLACK, VC_CYAN)},
{palidx_help_link_selected, "selected link", VCP(VC_LIGHT(VC_YELLOW), VC_BLUE)},
{palidx_help_link_unselected, "unselected link", VCP(VC_LIGHT(VC_YELLOW), VC_CYAN)}
};

/*
 *	INIT
 */

int init_help()
{
	int b=1;
	ht_registry_node_type rnt_pal=registry->have_node_type(rnt_palette_name);

/* HELP */
	b&=registry->create_subdir(palettekey"/"palclasskey_help);

	b&=create_pal(palclasskey_help, palkey_help_default, defpal_help, palcolors_help, rnt_pal);

	help=new ht_help();
	help->init();
	htinfostream=new ht_mem_stream();
	htinfostream->init();
	htinfostream->write(htinfo, strlen(htinfo));
	read_info_file(htinfostream);
	htinfostream->done();
	delete htinfostream;

	return b;
}

/*
 *	DONE
 */

int done_help()
{
	help->done();
	delete help;
	return 1;
}

