/* 
 *	HT Editor
 *	hthelp.h
 *
 *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __HTHELP_H__
#define __HTHELP_H__

#include "common.h"
#include "htdata.h"
#include "htobj.h"
#include "htstream.h"
#include "htformat.h"

int read_info_file(ht_stream *file);

/*
 *	CLASS ht_helpnode
 */

class ht_helpnode: public ht_data {
public:
	char *name;
	ht_string_list *content;

			void init(char *name);
	virtual 	void done();
/* debug */
	void debug_dump();
};

/*
 *	CLASS ht_helpfile
 */

class ht_helpfile: public ht_data {
protected:
	ht_list *nodes;
public:
	char *name;

			void init(char *name);
	virtual	void done();
/* new */
	ht_helpnode *findnode(char *name);
	void insertnode(ht_helpnode *node);
/* debug */
	void debug_dump();
};

/*
 *	CLASS ht_help
 */

class ht_help: public object {
public:
	ht_list *files;

			void init();
	virtual	void done();
/* new */
	ht_helpfile *findfile(char *name);
	void insertfile(ht_helpfile *file);
/* debug */
	void debug_dump();
};

/*
 *	CLASS ht_helpviewer
 */

class ht_helpviewer_status: public ht_data {
public:
	int cursorx, cursory;
	int ofsx, ofsy;
	char *filename, *nodename;

			void init();
	virtual	void done();
};

class ht_helpviewer: public ht_viewer {
protected:
	ht_string_list *content;
	int cursorx, cursory;
	int ofsx, ofsy;
	char *filename, *nodename;
	ht_scrollbar *scrollbar;
	char curlink_node_dest_s[128], curlink_file_dest_s[128];
	char *curlink_node_dest, *curlink_file_dest;
	int curline_vlen;
	ht_list *history;
/* new */
	int cursor_up(int n);
	int cursor_down(int n);
	int cursor_left(int n);
	int cursor_right(int n);
	void cursor_home();
	void cursor_end();
	void scroll_up(int n);
	void scroll_down(int n);
	void scroll_left(int n);
	void scroll_right(int n);

/* overwritten */
	virtual	char *defaultpalette();
	virtual	char *defaultpaletteclass();
/* new */
			void help_print(int x, int y, char *str);
			void reversenode();
			void update_scrollbar();
public:
			void init(bounds *b, ht_scrollbar *scrollbar);
	virtual	void done();
/* overwritten */
	virtual	void draw();
	virtual	void handlemsg(htmsg *msg);
		void releasefocus();
/* new */
			int gotonode(char *file, char *node);
};

/*
 *	INIT
 */

int init_help();

/*
 *	DONE
 */

int done_help();

#endif  /* !__HTHELP_H__ */
