/*
 *	HT Editor
 *	htpe.cc
 *
 *	Copyright (C) 1999, 2000, 2001 Stefan Weyergraf (stefan@weyergraf.de)
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 2 as
 *	published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "htapp.h"
#include "htnewexe.h"
#include "htpe.h"
#include "htpehead.h"
#include "htpeexp.h"
#include "htpeimp.h"
#include "htpedimp.h"
#include "htpeimg.h"
#include "htperes.h"
#include "htstream.h"
#include "tools.h"

#include <stdlib.h>
#include <string.h>

format_view_if *htpe_ifs[] = {
	&htpeheader_if,
	&htpeexports_if,
	&htpeimports_if,
	&htpedelayimports_if,
	&htperesources_if,
	&htpeimage_if,
	0
};

ht_view *htpe_init(bounds *b, ht_stream *file, ht_format_group *format_group)
{
	dword pemagic;
	STREAMOFS h=get_newexe_header_ofs(file);
	file->seek(h);
	file->read(&pemagic, 4);
	if (pemagic!=PE_MAGIC) return 0;

	ht_pe *g=new ht_pe();
	g->init(b, file, htpe_ifs, format_group, h);
	return g;
}

format_view_if htpe_if = {
	htpe_init,
	0
};

void ht_pe::init(bounds *b, ht_stream *file, format_view_if **ifs, ht_format_group *format_group, STREAMOFS header_ofs)
{
	ht_format_group::init(b, VO_BROWSABLE | VO_SELECTABLE | VO_RESIZE, DESC_PE, file, 0, 0, format_group);
	VIEW_DEBUG_NAME("ht_pe");

	LOG("%s: PE: found header at %08x", file->get_name(), header_ofs);
	ht_pe_shared_data *pe_shared=new ht_pe_shared_data;
	shared_data=pe_shared;
	pe_shared->header_ofs=header_ofs;

	pe_shared->exports.funcs=new ht_clist();
	pe_shared->exports.funcs->init();

	pe_shared->dimports.funcs=new ht_clist();
	pe_shared->dimports.funcs->init();

	pe_shared->dimports.libs=new ht_clist();
	pe_shared->dimports.libs->init();

	pe_shared->imports.funcs=new ht_clist();
	pe_shared->imports.funcs->init();

	pe_shared->imports.libs=new ht_clist();
	pe_shared->imports.libs->init();

	pe_shared->v_image=0;
	pe_shared->v_dimports=0;
	pe_shared->v_imports=0;
	pe_shared->v_exports=0;
	pe_shared->v_header=0;

/* read header */
	file->seek(header_ofs+4);
	file->read(&pe_shared->coffheader, sizeof pe_shared->coffheader);
	file->read(&pe_shared->opt_magic, sizeof pe_shared->opt_magic);
	file->seek(header_ofs+4+sizeof pe_shared->coffheader);
	switch (pe_shared->opt_magic) {
		case COFF_OPTMAGIC_PE32:
			file->read(&pe_shared->pe32.header, sizeof pe_shared->pe32.header);
			file->read(&pe_shared->pe32.header_nt, sizeof pe_shared->pe32.header_nt);
			break;
	}

/* read section headers */
	int os=pe_shared->coffheader.optional_header_size;
	pe_shared->sections.section_count=pe_shared->coffheader.section_count;
	
	file->seek(header_ofs+os+24);
	pe_shared->sections.sections=(COFF_SECTION_HEADER*)malloc(pe_shared->sections.section_count * sizeof *pe_shared->sections.sections);
	file->read(pe_shared->sections.sections, pe_shared->sections.section_count*sizeof *pe_shared->sections.sections);

	shared_data=pe_shared;

	ht_format_group::init_ifs(ifs);
}

void ht_pe::done()
{
	ht_format_group::done();
}

bool ht_pe::loc_enum_init(ht_format_loc *start_below)
{
	ht_pe_shared_data *sh=(ht_pe_shared_data*)shared_data;
	if ((!start_below) && (sh->opt_magic==COFF_OPTMAGIC_PE32)) {
		loc_enum=1;
		return 1;
	}
	return 0;
}

bool ht_pe::loc_enum_next(ht_format_loc *loc)
{
	ht_pe_shared_data *sh=(ht_pe_shared_data*)shared_data;
	if (loc_enum) {
		loc->name="pe";
		loc->start=sh->header_ofs;
// calc pe size
		UINT l=sizeof (COFF_HEADER) + sh->coffheader.optional_header_size;
// go through directories
		for (UINT i=0; i<16; i++) {
			STREAMOFS o;
			if (pe_rva_to_ofs(&sh->sections, sh->pe32.header_nt.directory[i].address, &o)) {
				UINT k=o+sh->pe32.header_nt.directory[i].size-sh->header_ofs;
				l=MAX(k, l);
			}
		}
// go through sections
		for (UINT i=0; i<sh->sections.section_count; i++) {
			UINT k=sh->sections.sections[i].data_offset+sh->sections.sections[i].data_size-sh->header_ofs;
			l=MAX(k, l);
		}
		loc->length=l;
		
		loc_enum=0;
		return 1;
	}
	return 0;
}

/*
 *	rva conversion routines
 */

bool pe_rva_to_ofs(pe_section_headers *section_headers, ADDR rva, STREAMOFS *ofs)
{
	COFF_SECTION_HEADER *s=section_headers->sections;
	for (UINT i=0; i<section_headers->section_count; i++) {
		if ((rva>=s->data_address) &&
		(rva<s->data_address+s->data_size)) {
			*ofs=rva-s->data_address+s->data_offset;
			return 1;
		}
		s++;
	}
	return 0;
}

bool pe_rva_to_section(pe_section_headers *section_headers, ADDR rva, int *section)
{
	COFF_SECTION_HEADER *s=section_headers->sections;
	for (UINT i=0; i<section_headers->section_count; i++) {
		if ((rva>=s->data_address) &&
		(rva<s->data_address+MAX(s->data_size, s->data_vsize))) {
			*section=i;
			return 1;
		}
		s++;
	}
	return 0;
}

bool pe_rva_is_valid(pe_section_headers *section_headers, ADDR rva)
{
	COFF_SECTION_HEADER *s=section_headers->sections;
	for (UINT i=0; i<section_headers->section_count; i++) {
		if ((rva>=s->data_address) &&
		(rva<s->data_address+MAX(s->data_size, s->data_vsize))) {
			return 1;
		}
		s++;
	}
	return 0;
}

bool pe_rva_is_physical(pe_section_headers *section_headers, ADDR rva)
{
	COFF_SECTION_HEADER *s=section_headers->sections;
	for (UINT i=0; i<section_headers->section_count; i++) {
		if ((rva>=s->data_address) &&
		(rva<s->data_address+s->data_size)) {
			return 1;
		}
		s++;
	}
	return 0;
}

/*
 *	ofs conversion routines
 */

bool pe_ofs_to_rva(pe_section_headers *section_headers, STREAMOFS ofs, ADDR *rva)
{
	COFF_SECTION_HEADER *s=section_headers->sections;
	for (UINT i=0; i<section_headers->section_count; i++) {
		if ((ofs>=s->data_offset) &&
		(ofs<s->data_offset+s->data_size)) {
			*rva=ofs-s->data_offset+s->data_address;
			return 1;
		}
		s++;
	}
	return 0;
}

bool pe_ofs_to_section(pe_section_headers *section_headers, STREAMOFS ofs, ADDR *section)
{
	COFF_SECTION_HEADER *s=section_headers->sections;
	for (UINT i=0; i<section_headers->section_count; i++) {
		if ((ofs>=s->data_offset) &&
		(ofs<s->data_offset+s->data_size)) {
			*section=i;
			return 1;
		}
		s++;
	}
	return 0;
}

bool pe_ofs_to_rva_and_section(pe_section_headers *section_headers, STREAMOFS ofs, ADDR *rva, ADDR *section)
{
	int r=pe_ofs_to_rva(section_headers, ofs, rva);
	if (r) {
		r=pe_ofs_to_section(section_headers, ofs, section);
	}
	return r;
}

bool pe_ofs_is_valid(pe_section_headers *section_headers, STREAMOFS ofs)
{
	ADDR rva;
	return pe_ofs_to_rva(section_headers, ofs, &rva);
}

