/*
 *   htkeyb.h - LINUX implementation
 */
 
/*
 * FIXME: keys with negative values are not (yet) usable under Linux...
 */ 

#ifndef __HTKEYB_H__
#define __HTKEYB_H__

#include "config.h"
#include CURSES_HDR
#undef clear
#undef move

#define K_Escape		'\e'
#define K_Return		0x0a
#define K_Space		 	0x20
#define K_Delete		KEY_DC
#define K_Insert		KEY_IL
#define K_BackSpace		KEY_BACKSPACE

#define HT_META_KEY(c) ((c) | 0x40000000)
#define HT_UNMETA_KEY(c) ((c) & (~0x40000000))
#define HT_IS_META_KEY(c) ((c) & 0x40000000)

// only works for A-Z !
#define HT_CONTROL_KEY(c) (char)(c-'A'+1)	

// my curses impl maps shift+f-x to f-(x+12), only works up to shift+f8...
#define HT_SHIFT_FKEY(n) (KEY_F0+12+n)

#define K_F1			KEY_F(1)
#define K_F2			KEY_F(2)
#define K_F3			KEY_F(3)
#define K_F4			KEY_F(4)
#define K_F5			KEY_F(5)
#define K_F6			KEY_F(6)
#define K_F7			KEY_F(7)
#define K_F8			KEY_F(8)
#define K_F9			KEY_F(9)
#define K_F10			KEY_F(10)
#define K_F11			KEY_F(11)
#define K_F12			KEY_F(12)
#define K_Home			KEY_HOME
#define K_End			KEY_END
#define K_Up			KEY_UP
#define K_Down			KEY_DOWN
#define K_Left			KEY_LEFT
#define K_Right			KEY_RIGHT
#define K_PageUp		KEY_PPAGE
#define K_PageDown		KEY_NPAGE
#define K_Tab			'\t'
#define K_BackTab		KEY_BTAB
#define K_Control_PageUp	HT_META_KEY(KEY_PPAGE)
#define K_Control_PageDown	HT_META_KEY(KEY_NPAGE)
#define K_Control_Left		-1
#define K_Control_Right		-2
#define K_Control_Up		-3
#define K_Control_Down		-4
#define K_Shift_Up		-5
#define K_Shift_Down		-6
#define K_Shift_Left		KEY_SLEFT
#define K_Shift_Right		KEY_SRIGHT
#define K_Shift_PageUp		-7
#define K_Shift_PageDown	-8
#define K_Shift_End		KEY_SEND
#define K_Shift_Delete		KEY_SDC
#define K_Shift_Insert		KEY_SIC
#define K_Shift_Home		KEY_SHOME
#define K_Control_F5		K_Alt_Q		/* hack ! */
#define K_Alt_F1		-10
#define K_Alt_F3		-11
#define K_Alt_F5		-12
#define K_Control_Insert	-14

#define K_Alt_1                HT_META_KEY('1')
#define K_Alt_2                HT_META_KEY('2')
#define K_Alt_3                HT_META_KEY('3')
#define K_Alt_4                HT_META_KEY('4')
#define K_Alt_5                HT_META_KEY('5')
#define K_Alt_6                HT_META_KEY('6')
#define K_Alt_7                HT_META_KEY('7')
#define K_Alt_8                HT_META_KEY('8')
#define K_Alt_9                HT_META_KEY('9')
#define K_Alt_0                HT_META_KEY('0')

#define K_Alt_A                HT_META_KEY('a')
#define K_Alt_B                HT_META_KEY('b')
#define K_Alt_C	               HT_META_KEY('c')
#define K_Alt_D                HT_META_KEY('d')
#define K_Alt_E                HT_META_KEY('e')
#define K_Alt_F                HT_META_KEY('f')
#define K_Alt_G	               HT_META_KEY('g')
#define K_Alt_H                HT_META_KEY('h')
#define K_Alt_I                HT_META_KEY('i')
#define K_Alt_J                HT_META_KEY('j')
#define K_Alt_K	               HT_META_KEY('k')
#define K_Alt_L                HT_META_KEY('l')
#define K_Alt_M                HT_META_KEY('m')
#define K_Alt_N                HT_META_KEY('n')
#define K_Alt_O	               HT_META_KEY('o')
#define K_Alt_P                HT_META_KEY('p')
#define K_Alt_Q                HT_META_KEY('q')
#define K_Alt_R                HT_META_KEY('r')
#define K_Alt_S	               HT_META_KEY('s')
#define K_Alt_T                HT_META_KEY('t')
#define K_Alt_U                HT_META_KEY('u')
#define K_Alt_V                HT_META_KEY('v')
#define K_Alt_W	               HT_META_KEY('w')
#define K_Alt_X                HT_META_KEY('x')
#define K_Alt_Y                HT_META_KEY('y')
#define K_Alt_Z                HT_META_KEY('z')

#define K_Control_A		HT_CONTROL_KEY('A')
#define K_Control_B		HT_CONTROL_KEY('B')
#define K_Control_C		HT_CONTROL_KEY('C')
#define K_Control_D		HT_CONTROL_KEY('D')
#define K_Control_E		HT_CONTROL_KEY('E')
#define K_Control_F		HT_CONTROL_KEY('F')
#define K_Control_G		HT_CONTROL_KEY('G')
#define K_Control_H		HT_CONTROL_KEY('H')
#define K_Control_I		HT_CONTROL_KEY('I')
#define K_Control_J		HT_CONTROL_KEY('J')
#define K_Control_K		HT_CONTROL_KEY('K')
#define K_Control_L		HT_CONTROL_KEY('L')
#define K_Control_M		HT_CONTROL_KEY('M')
#define K_Control_N		HT_CONTROL_KEY('N')
#define K_Control_O		HT_CONTROL_KEY('O')
#define K_Control_P		HT_CONTROL_KEY('P')
#define K_Control_Q		HT_CONTROL_KEY('Q')
#define K_Control_R		HT_CONTROL_KEY('R')
#define K_Control_S		HT_CONTROL_KEY('S')
#define K_Control_T		HT_CONTROL_KEY('T')
#define K_Control_U		HT_CONTROL_KEY('U')
#define K_Control_V		HT_CONTROL_KEY('V')
#define K_Control_W		HT_CONTROL_KEY('W')
#define K_Control_X		HT_CONTROL_KEY('X')
#define K_Control_Y		HT_CONTROL_KEY('Y')
#define K_Control_Z		HT_CONTROL_KEY('Z')

#define K_Shift_F1		HT_SHIFT_FKEY(1)
#define K_Shift_F2		HT_SHIFT_FKEY(2)
#define K_Shift_F3		HT_SHIFT_FKEY(3)
#define K_Shift_F4		HT_SHIFT_FKEY(4)
#define K_Shift_F5		HT_SHIFT_FKEY(5)
#define K_Shift_F6		HT_SHIFT_FKEY(6)
#define K_Shift_F7		HT_SHIFT_FKEY(7)
#define K_Shift_F8		HT_SHIFT_FKEY(8)
// these will probably not work
#define K_Shift_F9		HT_SHIFT_FKEY(9)
#define K_Shift_F10		HT_SHIFT_FKEY(10)
#define K_Shift_F11		HT_SHIFT_FKEY(11)
#define K_Shift_F12		HT_SHIFT_FKEY(12)

int htkey2alt(int key);
int htalt2key(int key);

int htkeypressed();
int htgetkey();

void ht_sleep(int sec, int usec);
#define HAVE_HT_SLEEP

int init_keyb();
int done_keyb();

#endif /* !__HTKEYB_H__ */
